package de.rossmann.iledocs;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.TreeMap;
import java.util.Vector;

/**
 * @author Mihael Schmidt
 *
 */
public class DataAccess 
{	
	private static SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");
	
	private static Connection dbConn = null;
    private static Statement dbStatement = null;
    
	public static void init(Properties p)
	{
        try
        {
	        DriverManager.registerDriver((Driver) Class.forName(p.getProperty("database.driver")).newInstance());
	        dbConn = DriverManager.getConnection(
	        		p.getProperty("database.url"), 
	        		p.getProperty("database.user"), 
	        		p.getProperty("database.password"));
	        dbStatement = dbConn.createStatement();
        }
        catch(Exception e)
        {
            System.err.println("[DatabaseAccess] Getting raw Statement object: " + e.getClass().getName() + " - " + e.getLocalizedMessage());
        }
	}
	
	public static void close()
	{
		try
		{
			dbStatement.close();
			dbConn.close();
		}
		catch(Exception e) 
		{ 
			System.err.println("Error during database clean up: " +  
					e.getClass() + " - " + e.getLocalizedMessage()); 
		}
	}
	
	private static Statement getDatabaseConnection()
	{
        return dbStatement;
	}

	public static TreeMap getProgramDescriptions(String library)
	{
		TreeMap pgms = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql ="SELECT base.docname AS name, base.srcfile AS sourcefile, TRIM(pgmdesc.value) AS desc, " +
						"base.user AS user, pgmdesc.attr as attribute " +  
					"FROM iledbase AS base LEFT OUTER JOIN iledbase AS pgmdesc " +
						"ON base.library = pgmdesc.library AND " +
							"base.srcfile = pgmdesc.srcfile AND " +
							"base.member = pgmdesc.member AND " +
							"pgmdesc.doctype = 'program' AND " +
							"pgmdesc.attr IN ('brief', 'deprecated', 'critical', 'sourceAttribute') " + 
					"WHERE base.library = '" + library + "' AND " +
							"base.doctype = 'program' AND " +
							"base.attr = 'programname'";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			while(rs.next())
			{
				String name = rs.getString("name");

				HashMap pgm = (HashMap) pgms.get(name);
				if (pgm == null)
				{
					// neue map anlegen
					pgm = new HashMap();
					pgm.put("user", rs.getString("user").trim());
					pgm.put("name", name);
					pgm.put("sourcefile", rs.getString("sourcefile").trim());
					pgms.put(name, pgm);
				}
				
				pgm.put(rs.getString("attribute").trim(), rs.getString("desc"));
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during libraries retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return pgms;
	}
	
	public static String getProgramName(String library, String file, String member)
	{
		String name = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"attr = 'programname'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				name = rs.getString("value").trim();
			}
			else
			{
				name = member;
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program name retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
			name = member;
		}
		
		return name;
	}
	
	public static String getBriefProgramDesc(String library, String file, String member)
	{
		String desc = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'brief'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				desc = rs.getString("value").trim();
			}
			else
			{
				desc = "";
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program brief desc retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
			desc = "";
		}
		
		return desc;
	}
	
	public static String getProgramDesc(String library, String file, String member)
	{
		String desc = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'desc'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				desc = rs.getString("value").trim();
			}
			else
			{
				desc = "";
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program desc retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
			desc = "";
		}
		
		return desc;
	}
	
	public static String getProgramDate(String library, String file, String member)
	{
		String date = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'date'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				date = rs.getString("value").trim();
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program date retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return date;
	}
	
	public static Collection getProgramTodos(String library, String file, String member)
	{
		Vector todos = new Vector();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr like 'todo%'";
		
		try
		{
			rs = stm.executeQuery(sql);
			while (rs.next())
			{
				todos.add(rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program todos retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return todos;
	}
	
	
	public static Collection getProgramInfos(String library, String file, String member)
	{
		Vector infos = new Vector();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr like 'info%'";
		
		try
		{
			rs = stm.executeQuery(sql);
			while (rs.next())
			{
				infos.add(rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program infos retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return infos;
	}
	
	
	public static HashMap getProgramLinks(String library, String file, String member)
	{
		HashMap links = new HashMap();
		HashMap tmp = new HashMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql ="SELECT attr, value FROM iledbase " +
					"WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr like 'link%' " +
					"ORDER BY attr";
		
		try
		{
			rs = stm.executeQuery(sql);
			while (rs.next())
			{
				String attr = rs.getString("attr").trim();
				if (attr.startsWith("linkDesc"))
				{
					String counter = attr.substring(8);
					tmp.put(counter, rs.getString("value").trim());
				}
				else if (attr.startsWith("linkURL"))
				{
					String counter = attr.substring(7);
					links.put(tmp.get(counter), rs.getString("value").trim());
				}
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program links retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return links;
	}
	
	public static Collection getProgramWarnings(String library, String file, String member)
	{
		Vector warnings = new Vector();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr like 'warning%'";
		
		try
		{
			rs = stm.executeQuery(sql);
			while (rs.next())
			{
				warnings.add(rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program warnings retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return warnings;
	}
	
	public static String getProgramAuthor(String library, String file, String member)
	{
		String author = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'author'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				author = rs.getString("value").trim();
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program author retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		return author;
	}
	
	
	public static String getProgramDeprecated(String library, String file, String member)
	{
		String deprecated = null;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'deprecated'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				deprecated = rs.getString("value").trim();
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program deprecation info retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return deprecated;
	}
	
	
	public static TreeMap getFiles(String library, String srcfile, String member)
	{
		TreeMap files = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT docname, attr, value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + srcfile + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'file' " +
					 "ORDER BY library, srcfile, member, doctype, docname";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			HashMap fileMap = new HashMap();
			Vector keywords = new Vector();
			String comp = "";
			String file = "";
			String attr = "";
			String value = "";
			
			fileMap.put("keywords", keywords);
			
			while (rs.next())
			{
				file = rs.getString("docname").trim();
				attr = rs.getString("attr").trim();
				value = rs.getString("value").trim();
				
				if (comp.length() == 0) comp = file;
				
				if (!comp.equals(file))
				{
					// new file -> store the old file in files
					files.put(fileMap.get("name"), fileMap);
					fileMap = new HashMap();
					keywords = new Vector();
					fileMap.put("keywords", keywords);
					comp = file;
				}
				
				if (attr.equals("keyword"))
					keywords.add(value);
				else
					fileMap.put(attr, value);
			}
			
			if (fileMap.size() > 1) 
				files.put(fileMap.get("name"), fileMap);
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during files retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return files;
	}
	
	public static Vector getExportedProcedures(String library)
	{
		Vector procs = new Vector();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = 
			"SELECT iledbase.srcfile as sourcefile, iledbase.member, " +
				"iledbase.docname, descbase.value as description, " +
				"iledbase.user as user, descbase.attr as attribute " +
			"FROM iledbase, iledbase as descbase " +
			"WHERE iledbase.library = '" + library + "' AND " +
				"iledbase.docname IN (SELECT docname FROM iledbase WHERE attr = 'exported' and library = '" + library + "') AND " + 
				"iledbase.attr = 'brief' AND iledbase.doctype = 'procedure' AND " +
				"iledbase.library = descbase.library AND " +
				"iledbase.srcfile = descbase.srcfile AND " +
				"iledbase.member = descbase.member AND " +
				"iledbase.doctype = descbase.doctype AND " +
				"iledbase.docname = descbase.docname AND " +
				"(descbase.attr = 'brief' or descbase.attr = 'deprecated') " +
			"ORDER BY iledbase.library, iledbase.srcfile, iledbase.member, iledbase.doctype, iledbase.docname";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			HashMap depMap = new HashMap();
			
			while (rs.next())
			{
				if (rs.getString("attribute").trim().equals("deprecated"))
				{
					depMap.put(rs.getString("docName").trim(), rs.getString("description").trim());
				}
				else
				{
					HashMap procMap = new HashMap();
					procMap.put("sourcefile", rs.getString("sourcefile").trim());
					procMap.put("program" , rs.getString("member").trim());
					procMap.put("procedure" , rs.getString("docName").trim());
					procMap.put("desc", rs.getString("description").trim());
					procMap.put("user" , rs.getString("user").trim());
					procs.add(procMap);	
				}
				
				// add deprecated info to procedure
				for (Iterator iter = procs.iterator(); iter.hasNext();)
				{
					HashMap procMap = (HashMap) iter.next();
					if (depMap.containsKey(procMap.get("procedure")))
					{
						procMap.put("deprecated", depMap.get(procMap.get("procedure")));
					}
				}
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during exported procedures retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return procs;
	}
	
	public static TreeMap getProcedures(String library, String srcfile, String member)
	{
		TreeMap procs = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT docname, attr, value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + srcfile + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'procedure' " +
					 "ORDER BY library, srcfile, member, doctype, docname";
		
		Vector infos = new Vector();
		Vector todos = new Vector();
		HashMap procMap = new HashMap();
		HashMap procParmDefMap = new HashMap();
		Vector procParmDefs = new Vector();
		String comp = "";
		String proc = "";
		String attr = "";
		String value = "";
		
		StringBuffer procParameter = new StringBuffer();
		
		int parmDefCount = 0;
		
		try
		{
			rs = stm.executeQuery(sql);
			
			while (rs.next())
			{				
				proc = rs.getString("docname").trim();
				attr = rs.getString("attr").trim();
				value = rs.getString("value").trim();
				
				if (comp.length() == 0) comp = proc;
				
				if (!comp.equals(proc))
				{
					// new proc -> store the old prc in procs
					for(int i = 1; i <= parmDefCount; i++)
					{
						procParmDefMap = new HashMap();
						procParmDefMap.put(
								"parmName",
								procMap.get("parm" + i));
						procParmDefMap.put(
								"parmDefinition",
								procMap.get("parmdef" + i));
						procParmDefs.add(procParmDefMap);
						
						if (i < parmDefCount)
						{
							procParameter.append(procMap.get("parmdefshort" + i) + ", ");
						}
						else
						{
							procParameter.append(procMap.get("parmdefshort" + i));
						}
					}
					
					if (parmDefCount > 0)
					{
						procMap.put("parmDefs", procParmDefs);
						procMap.put("procParameterShort", procParameter.toString());
					}
					procMap.put("parmCount", String.valueOf(parmDefCount));
					procMap.put("todos", todos);
					procMap.put("infos", infos);
					procs.put(procMap.get("procname"), procMap);
					procMap = new HashMap();
					procParmDefs = new Vector();
					parmDefCount = 0;
					procParameter = new StringBuffer();
					comp = proc;
					infos = new Vector();
					todos = new Vector();
				}
				
				if (attr.startsWith("parmdefshort"))
					parmDefCount++;
				else if (attr.startsWith("info"))
				{
					infos.add(value);
				}
				else if (attr.startsWith("todo"))
				{
					todos.add(value);
				}
					
				procMap.put(attr, value);
			}
			
			if (procMap.size() > 0) 
			{
				for(int i = 1; i <= parmDefCount; i++)
				{
					procParmDefMap = new HashMap();
					procParmDefMap.put(
							"parmName",
							procMap.get("parm" + i));
					procParmDefMap.put(
							"parmDefinition",
							procMap.get("parmdef" + i));
					procParmDefs.add(procParmDefMap);
					
					if (i < parmDefCount)
						procParameter.append(procMap.get("parmdefshort" + i) + ", ");
					else
						procParameter.append(procMap.get("parmdefshort" + i));
				}
				
				if (parmDefCount > 0)
				{
					procMap.put("parmDefs", procParmDefs);
					procMap.put("procParameterShort", procParameter.toString());
				}
				
				procMap.put("parmCount", String.valueOf(parmDefCount));
				procMap.put("todos", todos);
				procMap.put("infos", infos);
				
				procs.put(procMap.get("procname"), procMap);
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during procedures retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return procs;
	}
	
	
	public static TreeMap getProgramRevisions(String library, String srcfile, String member)
	{
		TreeMap revs = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT attr, value " + 
					 "FROM iledbase " + 
					 "WHERE library = '" + library +"' AND " +
					   "srcfile = '" + srcfile + "' AND " +
					   "member = '" + member + "' AND " + 
					   "docType = 'program' AND " +
					   "attr like 'rev%' " +
					 "ORDER BY attr";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			String attr = "";
			String key = "";
			Integer intx = null;
			
			while (rs.next())
			{
				attr = rs.getString("attr").trim();
				key = attr.substring(3, 7);
				intx = new Integer(attr.substring(7));
				
				if (!revs.containsKey(intx))
				{
					revs.put(intx, new HashMap());				
				}
				((HashMap) revs.get(intx)).put(key,rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program revisions retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return revs;
	}
	
	public static Collection getIncludes(String library, String srcfile, String member)
	{
		Vector includes = new Vector();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT attr, value " + 
					 "FROM iledbase " + 
					 "WHERE library = '" + library +"' AND " +
					   "srcfile = '" + srcfile + "' AND " +
					   "member = '" + member + "' AND " + 
					   "(attr like 'copy%')";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			while(rs.next())
			{
				String attr = rs.getString("attr").trim();
				int index = Integer.parseInt(attr.substring(8));
				if (includes.size() >= index)
				{
					// map allready there
					HashMap map = (HashMap) includes.get(index-1);
					map.put(attr.substring(4, 8), rs.getString("value"));
				}
				else
				{
					HashMap map = new HashMap();
					map.put(attr.substring(4, 8), rs.getString("value"));
					includes.add(map);
				}
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during includes/copybooks retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return includes;
	}
	
	
	
	public static boolean isProgramCritical(String library, String file, String member)
	{
		boolean retVal = false;
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql ="SELECT member, attr " + 
					"FROM iledbase " + 
					"WHERE doctype = 'program' AND " +
						"library = '" + library + "' AND " +
					    "srcfile = '" + file + "' AND " +
					    "member = '" + member + "' AND " + 
						"docname = '" + member + "' AND " + 
						"attr = 'critical'";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			if (rs.next())
			{
				retVal = true;
			}
			else
			{
				retVal = false;
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during critical program check: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return retVal;
	}
	
	public static TreeMap getProgramAuths(String library, String srcfile, String member)
	{
		TreeMap auths = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT attr, value " + 
					 "FROM iledbase " + 
					 "WHERE library = '" + library +"' AND " +
					   "srcfile = '" + srcfile + "' AND " +
					   "member = '" + member + "' AND " + 
					   "attr like 'authcode%'";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			String attr = "";
			String key = "";
			Integer intx = null;
			
			while (rs.next())
			{
				attr = rs.getString("attr").trim();
				if (attr.startsWith("authcodedesc"))
				{
					intx = new Integer(attr.substring(12));
					key = "desc";
				}
				else  // only authcode
				{
					intx = new Integer(attr.substring(8));
					key = "code";
				}
								
				if (!auths.containsKey(intx))
				{
					auths.put(intx, new HashMap());				
				}
				
				((HashMap) auths.get(intx)).put(key,rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program authcode retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return auths;
	}
	
	public static TreeMap getProgramConfigs(String library, String srcfile, String member)
	{
		TreeMap auths = new TreeMap();
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT attr, value " + 
					 "FROM iledbase " + 
					 "WHERE library = '" + library +"' AND " +
					   "srcfile = '" + srcfile + "' AND " +
					   "member = '" + member + "' AND " + 
					   "attr like 'config%'";
		
		try
		{
			rs = stm.executeQuery(sql);
			
			String attr = "";
			String key = "";
			Integer intx = null;
			
			while (rs.next())
			{
				attr = rs.getString("attr").trim();
				if (attr.startsWith("configDesc"))
				{
					intx = new Integer(attr.substring(10));
					key = "desc";
				}
				else  // configValue => key
				{
					intx = new Integer(attr.substring(11));
					key = "key";
				}
								
				if (!auths.containsKey(intx))
				{
					auths.put(intx, new HashMap());				
				}
				
				((HashMap) auths.get(intx)).put(key,rs.getString("value").trim());
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program configuration retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return auths;
	}
	
	public static String getProgramSourceAttribute(String library, String file, String member)
	{
		String attr = "";
		
		Statement stm = getDatabaseConnection();
		ResultSet rs = null;
		
		String sql = "SELECT value FROM iledbase " +
					 "WHERE library = '" + library + "' AND " +
					 	"srcfile = '" + file + "' AND " +
						"member = '" + member + "' AND " +
						"doctype = 'program' AND " +
						"attr = 'sourceAttribute'";
		
		try
		{
			rs = stm.executeQuery(sql);
			if (rs.next())
			{
				attr = rs.getString("value").trim();
			}
		}
		catch(Exception e)
		{
			System.err.println("[DatabaseAccess] Error during program source attribute retrieval: " + 
					e.getClass().getName() + " - " + e.getLocalizedMessage());
		}
		
		return attr;
	}
	
}